*** This is the file to draw random samples from a theory-inspired dgp  ***
*** ---Scenario 1---                                                    ***
*** First version: 12/06/2018                                           ***
*** This version:  04/07/2018                                           ***
*** Authors: Arun Advani, Toru Kitagawa and Tymon Sloczynski            ***
*** Written for Stata 11                                                ***


//all the usual setup stuff
*version control
version 11

*setup standard things
capture clear all
capture graph drop _all
capture log close

set seed 2018070401

set mem 300m
set more off


local NS = 1000 //number of samples

local OUTCOME       y
local TREATMENT     treated
local COVARIATES    x1
local COV1			x1
local NCOVS			2 //number of COVARIATES + 1

*outcome equation params
local beta_0  = 3
local beta_D  = .5
local beta_1  = .5
local beta_1D = 0
*error sd
local c       = 1
local sigma_0 = .5
local sigma_1 = `sigma_0'*sqrt(`c')
*propensity score params
local gamma_0 = .4
local gamma_1 = .1
*min and max for truncated normal
local a       = -4
local b       = 6

forvalues i=1/`NS' {
	drop _all
	
	set obs 1000
	
	*generate covariate
	gentrun double `COV1', left(`a') right(`b')
	
	*generate propensity score and treatment status
	gen p_treat 	= `gamma_0' + `gamma_1'*`COV1'
	gen random		= uniform()
	gen `TREATMENT' = p_treat>random
	
	*generate (possibly heteroscedastic) error, and outcome
	gen eps 		= invnormal(uniform()) * (`sigma_1'*`TREATMENT' + `sigma_0'*(1-`TREATMENT'))
	gen `OUTCOME'   = `beta_0' + `beta_D'*`TREATMENT' + `beta_1'*`COV1' + `beta_1D'*`TREATMENT'*`COV1' + eps
	
	drop p_treat random eps
	save "samples/sample`i'", replace
}

clear all
