clear all
set more off

set maxvar 32767

local NHLW		placebo_TS_20180718
local NSHLW		1000
local NBDM		structured_TS_20180719
local NSBDM		1000
local ESTM		b_lr b_ipw2

local FNHLW		01 02 03 04 05
local FNBDM		01 02 03 04 05

local ORIG		2_estimates

*************************



// PLACEBO

forvalues i=1/`NSHLW' {
	drop _all
	
	foreach FN of local FNHLW {
		capture use "placebo`FN'/`NHLW'_results_`i'", clear
	}
	capture keep b_* ave_*
	
	if _rc==0 {
		qui postfile biases_placebo str10(estimator) mbias ambias mse rmse variance sd using biases_placebo_`i', replace
		
		foreach var of local ESTM {
			gen bias_`var' = `var' - ave_effect
			egen mbias_`var' = mean(bias_`var')
			gen ambias_`var' = abs(mbias_`var')
			
			gen biassq_`var' = (`var' - ave_effect)^2
			egen mse_`var' = mean(biassq_`var')
			gen rmse_`var' = sqrt(mse_`var')
			
			qui sum `var'
			local variance = r(Var)
			local sd = r(sd)
			qui sum mbias_`var'
			local mbias = r(mean)
			qui sum ambias_`var'
			local ambias = r(mean)
			qui sum mse_`var'
			local mse = r(mean)
			qui sum rmse_`var'
			local rmse = r(mean)
			
			post biases_placebo ("`var'") (`mbias') (`ambias') (`mse') (`rmse') (`variance') (`sd')
		}
		
		postclose biases_placebo
	}
}

drop _all
forvalues i=1/`NSHLW' {
	capture append using biases_placebo_`i'
	qui save biases_placebo, replace
	erase biases_placebo_`i'.dta
}

use biases_placebo, clear
sort estimator
by estimator: egen mean_ambias = mean(ambias)
by estimator: egen mean_rmse = mean(rmse)
by estimator: egen mean_sd = mean(sd)
by estimator: egen sd_ambias = sd(ambias)
by estimator: egen sd_rmse = sd(rmse)
by estimator: egen sd_sd = sd(sd)
by estimator: gen n = _n
keep if n==1
keep estimator mean_* sd_*
save biases_placebo, replace



// STRUCTURED

forvalues i=1/`NSBDM' {
	drop _all
	
	foreach FN of local FNBDM {
		capture use "structured`FN'/`NBDM'_results_`i'", clear
	}
	capture keep b_* ave_*
	
	if _rc==0 {
		qui postfile biases_structured str10(estimator) mbias ambias mse rmse variance sd using biases_structured_`i', replace
		
		foreach var of local ESTM {
			gen bias_`var' = `var' - ave_effect
			egen mbias_`var' = mean(bias_`var')
			gen ambias_`var' = abs(mbias_`var')
			
			gen biassq_`var' = (`var' - ave_effect)^2
			egen mse_`var' = mean(biassq_`var')
			gen rmse_`var' = sqrt(mse_`var')
			
			qui sum `var'
			local variance = r(Var)
			local sd = r(sd)
			qui sum mbias_`var'
			local mbias = r(mean)
			qui sum ambias_`var'
			local ambias = r(mean)
			qui sum mse_`var'
			local mse = r(mean)
			qui sum rmse_`var'
			local rmse = r(mean)
			
			post biases_structured ("`var'") (`mbias') (`ambias') (`mse') (`rmse') (`variance') (`sd')
		}
		
		postclose biases_structured
	}
}

drop _all
forvalues i=1/`NSBDM' {
	capture append using biases_structured_`i'
	qui save biases_structured, replace
	erase biases_structured_`i'.dta
}

use biases_structured, clear
sort estimator
by estimator: egen mean_ambias = mean(ambias)
by estimator: egen mean_rmse = mean(rmse)
by estimator: egen mean_sd = mean(sd)
by estimator: egen sd_ambias = sd(ambias)
by estimator: egen sd_rmse = sd(rmse)
by estimator: egen sd_sd = sd(sd)
by estimator: gen n = _n
keep if n==1
keep estimator mean_* sd_*
save biases_structured, replace



// PRINT THE RESULTS

qui log using 8_appendix_tables, t replace

use biases_placebo, clear
format mean* sd* %30.3f
list

use biases_structured, clear
format mean* sd* %30.3f
list

qui log close



clear all
erase biases_placebo.dta
erase biases_structured.dta
