*** Estimates using samples from the original DGP                   ***
*** First version: 30/04/2018                                       ***
*** This version:  04/07/2018                                       ***
*** Authors: Arun Advani, Toru Kitagawa and Tymon Sloczynski        ***
*** Written for Stata 11                                            ***


//all the usual setup stuff
*version control
version 11

*setup standard things
capture clear all
capture log close
capture graph drop _all

*set processors 1

*allocate memory
set mem 300m

*get rid of scroll requirement
set more off

*install necessary programs
capture {
	ssc install fairlie, replace
	ssc install nnmatch, replace
	ssc install oaxaca, replace
	ssc install psmatch2, replace
	ssc install moremata, replace
}


//setup locals

local NSAMPLES		1000 //number of samples from the original dgp
local FOLDER		samples
local NAME			2_estimates

local OUTCOME       y
local TREATMENT     treated
local COVARIATES    x1
local COV1			x1
local NCOVS			2 //number of COVARIATES + 1

*************************



#delimit ;

quietly postfile results
	sample b_lr b_ipw2
	using `NAME', replace;

do "placebo/common_programs";

forvalues j=1/`NSAMPLES' {;
	timer on 1;
	quietly {;
	use "`FOLDER'/sample`j'", clear;
	
	*estimate propensity scores;
	regress `TREATMENT' `COVARIATES';
	predict double logps;
	
	*linear regression;
	reg `OUTCOME' `TREATMENT' `COVARIATES';
	scalar b_lr = _b[`TREATMENT'];
	
	*inverse probability weighting;
	capture ipw `OUTCOME' `TREATMENT' `COVARIATES';
	if _rc==0 {;
		scalar b_ipw2 = ipw2;
	};
	if _rc!=0 {;
		scalar b_ipw2 = .;
	};
	
	//STORE THE OUTPUT;
	post results
		(`j') (b_lr) (b_ipw2);
	};
	timer off 1;
	timer list 1;
};

postclose results;
clear all;
