#delimit ;

capture program drop difpscore; //This program is based on a chunk of code from psmatch2 (by Leuven and Sianesi).;
program define difpscore;
	syntax varname [if], obs(int) dif(string);
	qui g double `dif' = (`varlist' - `varlist'[`obs']) `if';
end;


capture program drop kweights; //This program is based on a chunk of code from psmatch2 (by Leuven and Sianesi).;
program define kweights;
	args kernel weight dif bwidth;
	
	if ("`kernel'"=="epan") {;
		qui g double `weight' = 1 - (`dif'/`bwidth')^2 if abs(`dif')<=`bwidth';
	};
	else if ("`kernel'"=="normal") {;
		qui g double `weight' = normalden(`dif'/`bwidth');
	};
	else if ("`kernel'"=="biweight") {;
		qui g double `weight' = (1 - (`dif'/`bwidth')^2)^2 if abs(`dif')<=`bwidth';
	};
	else if ("`kernel'"=="uniform") {;
		qui g double `weight' = 1 if abs(`dif')<=`bwidth';
	};
	else if ("`kernel'"=="tricube") {;
		qui g double `weight' = (1-abs(`dif'/`bwidth')^3)^3 if abs(`dif')<=`bwidth';
	};
	
	sum `weight', mean;
	replace `weight' = `weight'/r(sum);
end;


capture program drop mkernel; //This program is based on a chunk of code from psmatch2 (by Leuven and Sianesi).;
program define mkernel;
	syntax varlist(min=1) [, TReated(varlist) OUTcome(varlist) Kerneltype(string) BWidth(string) CROSSval(string)];
	tempname weight dif base;
	tempvar out;
	
	foreach v of varlist `outcome' {;
		qui g double _`v' = .;
	};

	count if `treated'==0;
	local N0 = r(N);
	local start = `N0' + 1;

	if "`crossval'"=="no" {;
		count if `treated'<=1;
		forvalues obs = `start'/`r(N)' {;
			difpscore `varlist' if `treated'==0, obs(`obs') dif(`dif');
			kweights `kerneltype' `weight' `dif' `bwidth';
	
			if ("`outcome'"!="") {;
				foreach v of varlist `outcome' {;
					sum `v' [aw=`weight'] if `treated'==0, mean;
					if (r(mean)!=.) replace _`v' = r(mean) in `obs';
				};
			};
			drop `weight' `dif';
		};
	
		foreach v of varlist `outcome' {;
			qui sum `v' if `treated'==1;
			scalar trt=r(mean);
			qui sum _`v' if `treated'==1;
			scalar ntrt=r(mean);
			scalar b_kernel = trt-ntrt;
			drop _`v'; scalar drop trt ntrt;
		};
	};
	
	else if "`crossval'"=="yes" {;
		count if `treated'==0;
		forvalues obs = 1/`r(N)' {;
			difpscore `varlist' if `treated'==0, obs(`obs') dif(`dif');
			replace `dif' = . in `obs';
			kweights `kerneltype' `weight' `dif' `bwidth';
	
			if ("`outcome'"!="") {;
				foreach v of varlist `outcome' {;
					sum `v' [aw=`weight'] if `treated'==0, mean;
					if (r(mean)!=.) replace _`v' = r(mean) in `obs';
				};
			};
			drop `weight' `dif';
		};
	
		foreach v of varlist `outcome' {;
			qui g double sqdiff_`v' = (`v' - _`v')^2;
			qui sum sqdiff_`v' if `treated'==0;
			scalar msd=r(mean);
			drop _`v' sqdiff_`v';
		};
	};
end;


capture program drop mllr; //This program is based on a chunk of code from psmatch2 (by Leuven and Sianesi).;
program define mllr;
	syntax varlist(min=1) [, TReated(varlist) OUTcome(varlist) Kerneltype(string) BWidth(string) CROSSval(string)];
	tempname weight dif base;
	tempvar out;
	
	foreach v of varlist `outcome' {;
		qui g double _`v' = .;
	};

	count if `treated'==0;
	local N0 = r(N);
	local start = `N0' + 1;

	if "`crossval'"=="no" {;
		count if `treated'<=1;
		forvalues obs = `start'/`r(N)' {;
			difpscore `varlist' if `treated'==0, obs(`obs') dif(`dif');
			kweights `kerneltype' `weight' `dif' `bwidth';
		
			if ("`outcome'"!="") {;
				foreach v of varlist `outcome' {;
					cap reg `v' `dif' [iw=`weight'] if `treated'==0;
					if (!_rc) replace _`v' = _b[_cons] in `obs';
				};
			};
			drop `weight' `dif';
		};
	
		foreach v of varlist `outcome' {;
			qui sum `v' if `treated'==1;
			scalar trt=r(mean);
			qui sum _`v' if `treated'==1;
			scalar ntrt=r(mean);
			scalar b_llr = trt-ntrt;
			drop _`v'; scalar drop trt ntrt;
		};
	};
	
	else if "`crossval'"=="yes" {;
		count if `treated'==0;
		forvalues obs = 1/`r(N)' {;
			difpscore `varlist' if `treated'==0, obs(`obs') dif(`dif');
			replace `dif' = . in `obs';
			kweights `kerneltype' `weight' `dif' `bwidth';
		
			if ("`outcome'"!="") {;
				foreach v of varlist `outcome' {;
					cap reg `v' `dif' [iw=`weight'] if `treated'==0;
					if (!_rc) replace _`v' = _b[_cons] in `obs';
				};
			};
			drop `weight' `dif';
		};
	
		foreach v of varlist `outcome' {;
			qui g double sqdiff_`v' = (`v' - _`v')^2;
			qui sum sqdiff_`v' if `treated'==0;
			scalar msd=r(mean);
			drop _`v' sqdiff_`v';
		};
	};
end;


capture program drop mllog; //This program is based on a chunk of code from psmatch2 (by Leuven and Sianesi).;
program define mllog;
	syntax varlist(min=1) [, TReated(varlist) OUTcome(varlist) Kerneltype(string) BWidth(string) CROSSval(string)];
	tempname weight dif base;
	tempvar out;
	
	foreach v of varlist `outcome' {;
		qui g double _`v' = .;
	};

	count if `treated'==0;
	local N0 = r(N);
	local start = `N0' + 1;

	if "`crossval'"=="no" {;
		count if `treated'<=1;
		forvalues obs = `start'/`r(N)' {;
			difpscore `varlist' if `treated'==0, obs(`obs') dif(`dif');
			kweights `kerneltype' `weight' `dif' `bwidth';
		
			if ("`outcome'"!="") {;
				foreach v of varlist `outcome' {;
					cap logit `v' `dif' [iw=`weight'] if `treated'==0, tolerance(1e-4) iterate(100);
					if (!_rc) replace _`v' = invlogit(_b[_cons]) in `obs';
				};
			};
			drop `weight' `dif';
		};
	
		foreach v of varlist `outcome' {;
			qui sum `v' if `treated'==1;
			scalar trt=r(mean);
			qui sum _`v' if `treated'==1;
			scalar ntrt=r(mean);
			scalar b_llog = trt-ntrt;
			drop _`v'; scalar drop trt ntrt;
		};
	};
	
	else if "`crossval'"=="yes" {;
		count if `treated'==0;
		forvalues obs = 1/`r(N)' {;
			difpscore `varlist' if `treated'==0, obs(`obs') dif(`dif');
			replace `dif' = . in `obs';
			kweights `kerneltype' `weight' `dif' `bwidth';
		
			if ("`outcome'"!="") {;
				foreach v of varlist `outcome' {;
					cap logit `v' `dif' [iw=`weight'] if `treated'==0, tolerance(1e-4) iterate(100);
					if (!_rc) replace _`v' = invlogit(_b[_cons]) in `obs';
				};
			};
			drop `weight' `dif';
		};
	
		foreach v of varlist `outcome' {;
			qui g double sqdiff_`v' = (`v' - _`v')^2;
			qui sum sqdiff_`v' if `treated'==0;
			scalar msd=r(mean);
			drop _`v' sqdiff_`v';
		};
	};
end;


capture program drop ipw;
program define ipw; args OUTCOME TREATMENT COVARIATES;
	sum `OUTCOME' if `TREATMENT' == 1; scalar ym_treated = r(mean);
	sum `TREATMENT'; scalar number = r(N); scalar pr_treated = r(mean); scalar pr_nontreated = 1-r(mean);
	gen denom2 = ((1-`TREATMENT')*logps)/(1-logps); sum denom2 if `TREATMENT' == 0; scalar m_denom2 = r(mean);
	gen c_i_summand = (1-(logps/pr_treated)*((1-`TREATMENT')/(1-logps))); sum c_i_summand; scalar m_c_i_summand = r(mean);
	gen c_i_summand2 = c_i_summand^2; sum c_i_summand2; scalar m_c_i_summand2 = r(mean);
	gen c_i = ((1-(logps/pr_treated)*((1-`TREATMENT')/(1-logps)))*m_c_i_summand)/m_c_i_summand2;
	gen denom3 = (((1-`TREATMENT')*logps)/(1-logps))*(1-c_i); sum denom3 if `TREATMENT' == 0; scalar m_denom3 = r(mean);
	gen y_rewght1 = (`OUTCOME'*(1-`TREATMENT'))*(logps/(1-logps))*(pr_nontreated/pr_treated); sum y_rewght1 if `TREATMENT' == 0; scalar ym_rewght1 = r(mean);
	gen y_rewght2 = (`OUTCOME'*(1-`TREATMENT'))*(logps/(1-logps))*(1/m_denom2); sum y_rewght2 if `TREATMENT' == 0; scalar ym_rewght2 = r(mean);
	gen y_rewght3 = (`OUTCOME'*(1-`TREATMENT'))*(logps/(1-logps))*(1-c_i)*(1/m_denom3); sum y_rewght3 if `TREATMENT' == 0; scalar ym_rewght3 = r(mean);
	scalar ipw1 = ym_treated - ym_rewght1;
	scalar ipw2 = ym_treated - ym_rewght2;
	scalar ipw3 = ym_treated - ym_rewght3;
	drop denom2 c_i_summand c_i_summand2 c_i denom3 y_rewght1 y_rewght2 y_rewght3;
	scalar drop ym_rewght2 ym_rewght1 m_denom2 pr_nontreated pr_treated number ym_treated ym_rewght3 m_denom3 m_c_i_summand2 m_c_i_summand;
end;


capture program drop drobust;
program define drobust; args OUTCOME TREATMENT COVARIATES LOGIT;
	gen ipw_wght = logps/(1-logps);
	if "`LOGIT'"=="yes" logit `OUTCOME' `COVARIATES' [pw=ipw_wght] if `TREATMENT' == 0, tolerance(1e-4) iterate(100);
	else if "`LOGIT'"=="no" reg `OUTCOME' `COVARIATES' [pw=ipw_wght] if `TREATMENT' == 0;
	predict out_wght; gen diff = `OUTCOME' - out_wght; sum diff if `TREATMENT' == 1;
    scalar drob = r(mean); drop ipw_wght out_wght diff;
end;


capture program drop melly;
program define melly; args OUTCOME TREATMENT BAND LOGIT GAUSS;
	if "`LOGIT'"=="yes" & "`GAUSS'"=="no" locreg4 `OUTCOME' if `TREATMENT' == 0, c(logps) b(`BAND') g(fitted) eval(`TREATMENT') logit;
	else if "`LOGIT'"=="no" & "`GAUSS'"=="no" locreg4 `OUTCOME' if `TREATMENT' == 0, c(logps) b(`BAND') g(fitted) eval(`TREATMENT');
	else if "`LOGIT'"=="yes" & "`GAUSS'"=="yes" locreg4 `OUTCOME' if `TREATMENT' == 0, c(logps) b(`BAND') g(fitted) eval(`TREATMENT') k(gaussian) logit;
	else if "`LOGIT'"=="no" & "`GAUSS'"=="yes" locreg4 `OUTCOME' if `TREATMENT' == 0, c(logps) b(`BAND') g(fitted) eval(`TREATMENT') k(gaussian);	
	gen diff = `OUTCOME' - fitted; sum diff if `TREATMENT' == 1;
    scalar lcl = r(mean); drop diff fitted;
end;


capture program drop comsup;
program define comsup; args TREATMENT;
	sum logps if `TREATMENT' == 0; drop if logps>r(max);
	sum logps if `TREATMENT' == 0; drop if logps<r(min);
end;


capture program drop comsup2;
program define comsup2; args TREATMENT WITHIN;
	sum logps if `TREATMENT' == 1; scalar max1 = r(max);
	sum logps if `TREATMENT' == 1; scalar min1 = r(min);
	sum logps if `TREATMENT' == 0; scalar max0 = r(max);
	sum logps if `TREATMENT' == 0; scalar min0 = r(min);
	if max1>max0 scalar max=max0; else scalar max=max1;
	if min1<min0 scalar min=min0; else scalar min=min1;
	replace `WITHIN'=0 if logps>max | logps<min;
	replace `WITHIN'=1 if `WITHIN'==.;
end;
