clear all
set more off

set maxvar 32767

local NHLW		placebo_TS_20180718
local NSHLW		1000
local NBDM		structured_TS_20180719
local NSBDM		1000
local ESTM		b_lr b_ipw2

local FNHLW		01 02 03 04 05
local FNBDM		01 02 03 04 05

local ORIG		2_estimates

*************************



// PLACEBO

forvalues i=1/`NSHLW' {
	drop _all
	
	foreach FN of local FNHLW {
		capture use "placebo`FN'/`NHLW'_results_`i'", clear
	}
	capture keep b_* ave_*
	
	if _rc==0 {
		qui postfile biases_placebo str10(estimator) mbias`i' ambias`i' mse`i' rmse`i' variance`i' using biases_placebo_`i', replace
		
		foreach var of local ESTM {
			gen bias_`var' = `var' - ave_effect
			egen mbias_`var' = mean(bias_`var')
			gen ambias_`var' = abs(mbias_`var')
			
			gen biassq_`var' = (`var' - ave_effect)^2
			egen mse_`var' = mean(biassq_`var')
			gen rmse_`var' = sqrt(mse_`var')
			
			qui sum `var'
			local variance = r(Var)
			qui sum mbias_`var'
			local mbias = r(mean)
			qui sum ambias_`var'
			local ambias = r(mean)
			qui sum mse_`var'
			local mse = r(mean)
			qui sum rmse_`var'
			local rmse = r(mean)
			
			post biases_placebo ("`var'") (`mbias') (`ambias') (`mse') (`rmse') (`variance')
		}
		
		postclose biases_placebo
		
		use biases_placebo_`i', clear
		sort ambias`i'
		gen rank_ambias`i'=_n
		sort mse`i'
		gen rank_mse`i'=_n
		qui save biases_placebo_`i', replace
	}
}

use 5_biases_true, clear
qui egen minambias = min(ambias)
qui gen diff_ambias = ambias - minambias
qui egen minrmse = min(rmse)
qui gen diff_rmse = rmse - minrmse
drop minambias minrmse
forvalues i=1/`NSHLW' {
	capture merge 1:1 estimator using biases_placebo_`i'
	if _rc==0 {
		drop _merge
		qui save biases_placebo, replace
		erase biases_placebo_`i'.dta
	}
}

use biases_placebo, clear
qui postfile results_placebo corr_rank_ambias corr_rank_mse min_ambias diff_ambias min_rmse diff_rmse using 6_results_placebo, replace
forvalues i=1/`NSHLW' {
	capture novarabbrev pwcorr rank_ambias rank_ambias`i'
	if _rc==0 {
		qui pwcorr rank_ambias rank_ambias`i'
		local corr_rank_ambias = r(rho)
		qui pwcorr rank_mse rank_mse`i'
		local corr_rank_mse = r(rho)
		qui sum ambias if rank_ambias==1
		local min_ambias = r(mean)
		qui sum diff_ambias if rank_ambias`i'==1
		local diff_ambias = r(mean)
		qui sum rmse if rank_mse==1
		local min_rmse = r(mean)
		qui sum diff_rmse if rank_mse`i'==1
		local diff_rmse = r(mean)
		
		post results_placebo (`corr_rank_ambias') (`corr_rank_mse') (`min_ambias') (`diff_ambias') (`min_rmse') (`diff_rmse')
	}
}
postclose results_placebo



// STRUCTURED

forvalues i=1/`NSBDM' {
	drop _all
	
	foreach FN of local FNBDM {
		capture use "structured`FN'/`NBDM'_results_`i'", clear
	}
	capture keep b_* ave_*
	
	if _rc==0 {
		qui postfile biases_structured str10(estimator) mbias`i' ambias`i' mse`i' rmse`i' variance`i' using biases_structured_`i', replace
		
		foreach var of local ESTM {
			gen bias_`var' = `var' - ave_effect
			egen mbias_`var' = mean(bias_`var')
			gen ambias_`var' = abs(mbias_`var')
			
			gen biassq_`var' = (`var' - ave_effect)^2
			egen mse_`var' = mean(biassq_`var')
			gen rmse_`var' = sqrt(mse_`var')
			
			qui sum `var'
			local variance = r(Var)
			qui sum mbias_`var'
			local mbias = r(mean)
			qui sum ambias_`var'
			local ambias = r(mean)
			qui sum mse_`var'
			local mse = r(mean)
			qui sum rmse_`var'
			local rmse = r(mean)
			
			post biases_structured ("`var'") (`mbias') (`ambias') (`mse') (`rmse') (`variance')
		}
		
		postclose biases_structured
		
		use biases_structured_`i', clear
		sort ambias`i'
		gen rank_ambias`i'=_n
		sort mse`i'
		gen rank_mse`i'=_n
		qui save biases_structured_`i', replace
	}
}

use 5_biases_true, clear
qui egen minambias = min(ambias)
qui gen diff_ambias = ambias - minambias
qui egen minrmse = min(rmse)
qui gen diff_rmse = rmse - minrmse
drop minambias minrmse
forvalues i=1/`NSBDM' {
	capture merge 1:1 estimator using biases_structured_`i'
	if _rc==0 {
		drop _merge
		qui save biases_structured, replace
		erase biases_structured_`i'.dta
	}
}

use biases_structured, clear
qui postfile results_structured corr_rank_ambias corr_rank_mse min_ambias diff_ambias min_rmse diff_rmse using 6_results_structured, replace
forvalues i=1/`NSBDM' {
	capture novarabbrev pwcorr rank_ambias rank_ambias`i'
	if _rc==0 {
		qui pwcorr rank_ambias rank_ambias`i'
		local corr_rank_ambias = r(rho)
		qui pwcorr rank_mse rank_mse`i'
		local corr_rank_mse = r(rho)
		qui sum ambias if rank_ambias==1
		local min_ambias = r(mean)
		qui sum diff_ambias if rank_ambias`i'==1
		local diff_ambias = r(mean)
		qui sum rmse if rank_mse==1
		local min_rmse = r(mean)
		qui sum diff_rmse if rank_mse`i'==1
		local diff_rmse = r(mean)
		
		post results_structured (`corr_rank_ambias') (`corr_rank_mse') (`min_ambias') (`diff_ambias') (`min_rmse') (`diff_rmse')
	}
}
postclose results_structured



clear all
erase biases_placebo.dta
erase biases_structured.dta
