*** This is the file to draw random samples (without replacement)   ***
***    from the NSW-CPS dataset (LaLonde 1986, DW 1999, ST 2005)    ***
*** Version of the dataset (DW/ST): Smith & Todd                    ***
*** Target group from NSW (ctr/exp): control group                  ***
*** First version: 30/04/2018                                       ***
*** This version:  24/05/2018                                       ***
*** Authors: Arun Advani, Toru Kitagawa and Tymon Sloczynski        ***
*** Written for Stata 11                                            ***


//all the usual setup stuff
*version control
version 11

*setup standard things
capture clear all
capture graph drop _all
capture log close

set seed 20180524

*allocate memory
set mem 300m

*get rid of scroll requirement
set more off


//setup locals
local FOLDER		samples //target folder
local TARGET		nswera_control //target dataset
local NONTARGET		nswera_treated //another dataset from either ST or DW
local COMPARISON	cps_controls //comparison dataset

local NS			1000 //number of samples
local NT			100 //number of treated units per sample
local NC			1900 //number of control units per sample

local TREATMENT     treated
local COVARIATES    age age2 age3 educ educ2 nodegree married black hispanic re74 re75 u74 u75 re74ed



*************************

//restrict our attention to the overlap-screened portion of the CPS dataset
use "original_data/`TARGET'", clear
append using "original_data/`NONTARGET'"
capture rename hisp hispanic
capture drop dwincl early_ra sample
replace `TREATMENT'=1 if `TREATMENT'==0
append using "original_data/`COMPARISON'"

gen u74 = 1 if re74==0
replace u74 = 0 if re74!=0
gen u75 = 1 if re75==0
replace u75 = 0 if re75!=0
gen age2 = age^2
gen age3 = age^3
gen educ2 = educ^2
gen re74ed = educ*re74

logit `TREATMENT' `COVARIATES', asis
predict double logps
	
sum logps if `TREATMENT'==1, d
drop if logps>r(max) | logps<r(min)
keep if `TREATMENT'==0
drop u74-logps
save `COMPARISON'_temp, replace



//draw samples
forvalues i=1/`NS' {
	use "original_data/`TARGET'", clear
	capture rename hisp hispanic
	capture drop dwincl early_ra sample
	replace `TREATMENT'=1 if `TREATMENT'==0
	gen random=uniform()
	sort random
	keep in 1/`NT'
	drop random
	save temp_T, replace
	
	use `COMPARISON'_temp, clear
	gen random=uniform()
	sort random
	keep in 1/`NC'
	drop random
	save temp_C, replace
	
	append using temp_T, nolabel nonotes
	save "`FOLDER'/sample`i'"
}

clear all
erase temp_T.dta
erase temp_C.dta
erase `COMPARISON'_temp.dta
