clear all
set more off

local NN			4

local TARGET		nswre74_treated //target dataset
local NONTARGET		nswre74_control //another dataset from either ST or DW
local COMPARISON	cps_controls //comparison dataset

local OUTCOME       re78
local TREATMENT     treated
local COVARIATES    age age2 age3 educ educ2 nodegree married black hispanic re74 re75 u74 u75 re74ed

*************************



use "original_data/`TARGET'", clear
append using "original_data/`NONTARGET'"
capture rename hisp hispanic
capture drop dwincl early_ra sample
replace `TREATMENT'=1 if `TREATMENT'==0
append using "original_data/`COMPARISON'"

gen u74 = 1 if re74==0
replace u74 = 0 if re74!=0
gen u75 = 1 if re75==0
replace u75 = 0 if re75!=0
gen age2 = age^2
gen age3 = age^3
gen educ2 = educ^2
gen re74ed = educ*re74

logit `TREATMENT' `COVARIATES', asis
predict double logps
	
sum logps if `TREATMENT'==1, d
drop if logps>r(max) | logps<r(min)
keep if `TREATMENT'==0
drop u74-logps
save `COMPARISON'_temp, replace



use "original_data/`TARGET'", clear
capture rename hisp hispanic
capture drop dwincl early_ra sample
append using `COMPARISON'_temp

gen u74 = 1 if re74==0
replace u74 = 0 if re74!=0
gen u75 = 1 if re75==0
replace u75 = 0 if re75!=0
gen age2 = age^2
gen age3 = age^3
gen educ2 = educ^2
gen re74ed = educ*re74

gen id=_n
order id, first
save cps_temp, replace

teffects nnmatch (`OUTCOME' `COVARIATES') (`TREATMENT'), generate(stub) nn(`NN')

qui sum treated
local N=r(N)

local ch=1
local st=0
while `ch'>0 {
	local st=`st'+1
	capture sum stub`st'
	if _rc!=0 local ch=0
}
local st=`st'-1
di "max(stub): " `st'


forvalues i=1/`N' {
	qui postfile temp `OUTCOME' using temp`i', replace
	forvalues j=1/`st' {
		qui sum stub`j' in `i'
		local id=r(mean)
		
		if `id'<. {
			qui sum `OUTCOME' in `id'
			local mean=r(mean)
			post temp (`mean')
		}
		
		else post temp (`id')
	}
	postclose temp
}


qui postfile match id cfact using cps_match, replace
forvalues i=1/`N' {
	use temp`i', clear
	qui sum `OUTCOME'
	local mean=r(mean)
	post match (`i') (`mean')
}
postclose match


use cps_temp, clear
merge 1:1 id using cps_match

drop id
qui sum _merge
if r(sd)==0 drop _merge

gen `OUTCOME'_t = `TREATMENT'*`OUTCOME' + (1-`TREATMENT')*cfact
gen `OUTCOME'_c = `TREATMENT'*cfact + (1-`TREATMENT')*`OUTCOME'
gen te = `OUTCOME'_t - `OUTCOME'_c
logit `TREATMENT' `COVARIATES', asis
predict pscore, xb
order `OUTCOME' `TREATMENT' `OUTCOME'_t `OUTCOME'_c te pscore, first
drop cfact
save 0_population, replace

sum te
teffects nnmatch (`OUTCOME' `COVARIATES') (`TREATMENT'), ate nn(`NN')

sum te if `TREATMENT'
teffects nnmatch (`OUTCOME' `COVARIATES') (`TREATMENT'), atet nn(`NN')


clear all
erase cps_temp.dta
erase cps_match.dta
erase `COMPARISON'_temp.dta
forvalues i=1/`N' {
	erase temp`i'.dta
}
