*** This is the file to draw random samples (without replacement)   ***
***    from the NSW-CPS dataset (LaLonde 1986, DW 1999, ST 2005)    ***
*** Prepared for the pseudo-true population based on NN matching    ***
*** First version: 22/05/2018                                       ***
*** This version:  24/05/2018                                       ***
*** Authors: Arun Advani, Toru Kitagawa and Tymon Sloczynski        ***
*** Written for Stata 11                                            ***


//all the usual setup stuff
*version control
version 11

*setup standard things
capture clear all
capture graph drop _all
capture log close

set seed 2018052425

*allocate memory / get rid of scroll requirement
set mem 300m
set more off


//setup locals
local DATA			0_population
local FOLDER		samples //target folder

local NS			1000 //number of samples
local NOBS			2000 //number of units per sample

local ORIG			NO //whether retain the original p(treat)
local PRTREAT		0.25 //new p(treat)

*************************



//draw samples
forvalues i=1/`NS' {
	use `DATA', clear
	capture drop treated re78
	gen random = uniform()
	sort random
	keep in 1/`NOBS'
	drop random
	
	gen u_logistic = -1*(sqrt(3)/_pi)*ln((1/uniform())-1)
	replace pscore = pscore + u_logistic
	
	if "`ORIG'"=="NO" {
		local PRTREAT2 = 100-(100*`PRTREAT')
		centile pscore, c(`PRTREAT2')
		scalar lambda = -1*r(c_1)
		replace pscore = pscore + lambda
	}
	
	gen treated = pscore>0
	gen re78 = treated*re78_t + (1-treated)*re78_c
	keep treated age educ black hispanic married nodegree re74 re75 re78
	save "`FOLDER'/sample`i'"
}

clear all
