*** This is the file to calculate the true value of the ATT    ***
***    in the pseudo-true population based on NN matching      ***
*** First version: 20/06/2018                                  ***
*** This version:  23/06/2018                                  ***
*** Authors: Arun Advani, Toru Kitagawa and Tymon Sloczynski   ***
*** Written for Stata 11                                       ***


//all the usual setup stuff
*version control
version 11

*setup standard things
capture clear all
capture graph drop _all
capture log close

set seed 20180623

*allocate memory / get rid of scroll requirement
set mem 300m
set more off


//setup locals
local DATA			0_population
local TEMP			0_population_temp

local NS			10000 //number of samples
local NOBS			2000 //number of units per sample

local ORIG			NO //whether retain the original p(treat)
local PRTREAT		0.25 //new p(treat)

*************************



use `DATA', clear
gen id = _n
order id, first
save `TEMP', replace

//draw samples
forvalues i=1/`NS' {
	timer on 1
	use `TEMP', clear
	capture drop treated re78
	gen random = uniform()
	sort random
	qui keep in 1/`NOBS'
	drop random
	
	gen u_logistic = -1*(sqrt(3)/_pi)*ln((1/uniform())-1)
	qui replace pscore = pscore + u_logistic
	
	if "`ORIG'"=="NO" {
		local PRTREAT2 = 100-(100*`PRTREAT')
		qui centile pscore, c(`PRTREAT2')
		scalar lambda = -1*r(c_1)
		qui replace pscore = pscore + lambda
	}
	
	gen treated = pscore>0
	keep id treated
	qui save sample`i'
	timer off 1
	timer list 1
}


//combine samples
drop _all
forvalues i=1/`NS' {
	timer on 2
	append using sample`i'
	timer off 2
	timer list 2
}
sort id
by id: egen p = mean(treated)
by id: gen n=_n
by id: gen N=_N
keep if n==1
drop n N treated


//merge with population
merge 1:1 id using `TEMP', nogen
save `TEMP', replace

gen product = p*te
qui sum product
local MEAN = r(mean)
local ATT = `MEAN'/`PRTREAT'


//save the value of the target parameter
drop _all
set obs 1

gen effect = `ATT'
save 4_effects_true, replace
clear all


//delete temp files
forvalues i=1/`NS' {
	timer on 3
	erase sample`i'.dta
	timer off 3
	timer list 3
}
erase `TEMP'.dta
