clear all
set more off

set maxvar 32767
*set seed 20180816

*local NSRND	1000
local NHLW		placebo_TS_20180524
local NSHLW		1000
local NBDM		structured_TS_20180524
local NSBDM		1000
local NBTS		bootstrap_TS_20180524
local NSBTS		1000
local ESTM		b_lr b_oaxaca b_dr b_kern_u b_ipw2 b_psm1 b_psmbc1

local FNHLW		01 02 03 04 05 06 07 08
local FNBDM		01 02 03 04 05 06 07 08 09 10
local FNBTS		01 02 03 04 05 06 07 08 09 10

local ORIG		2_estimates

*************************



// PLACEBO

forvalues i=1/`NSHLW' {
	drop _all
	
	foreach FN of local FNHLW {
		capture use "placebo`FN'/`NHLW'_results_`i'", clear
	}
	capture keep b_* ave_*
	
	if _rc==0 {
		qui postfile biases_placebo str10(estimator) mbias`i' ambias`i' mse`i' rmse`i' variance`i' using biases_placebo_`i', replace
		
		foreach var of local ESTM {
			gen bias_`var' = `var' - ave_effect
			egen mbias_`var' = mean(bias_`var')
			gen ambias_`var' = abs(mbias_`var')
			
			gen biassq_`var' = (`var' - ave_effect)^2
			egen mse_`var' = mean(biassq_`var')
			gen rmse_`var' = sqrt(mse_`var')
			
			qui sum `var'
			local variance = r(Var)
			qui sum mbias_`var'
			local mbias = r(mean)
			qui sum ambias_`var'
			local ambias = r(mean)
			qui sum mse_`var'
			local mse = r(mean)
			qui sum rmse_`var'
			local rmse = r(mean)
			
			post biases_placebo ("`var'") (`mbias') (`ambias') (`mse') (`rmse') (`variance')
		}
		
		postclose biases_placebo
		
		use biases_placebo_`i', clear
		sort ambias`i'
		gen rank_ambias`i'=_n
		sort mse`i'
		gen rank_mse`i'=_n
		sort variance`i'
		gen rank_var`i'=_n
		save biases_placebo_`i', replace
	}
}

use 5_biases_true, clear
qui egen minambias = min(ambias)
qui gen diff_ambias = ambias - minambias
qui egen minmse = min(mse)
qui gen diff_mse = mse - minmse
qui egen minvar = min(variance)
qui gen diff_var = variance - minvar
drop minambias minmse minvar
forvalues i=1/`NSHLW' {
	capture merge 1:1 estimator using biases_placebo_`i'
	if _rc==0 {
		drop _merge
		qui save biases_placebo, replace
		erase biases_placebo_`i'.dta
	}
}

use biases_placebo, clear
qui postfile results_placebo corr_mbias corr_ambias corr_mse corr_rmse corr_var tau_ambias tau_mse tau_var min_ambias diff_ambias min_mse diff_mse min_var diff_var using 6_results_placebo, replace
forvalues i=1/`NSHLW' {
	capture novarabbrev pwcorr mbias mbias`i'
	if _rc==0 {
		qui pwcorr mbias mbias`i'
		local corr_mbias = r(rho)
		qui pwcorr ambias ambias`i'
		local corr_ambias = r(rho)
		qui pwcorr mse mse`i'
		local corr_mse = r(rho)
		qui pwcorr rmse rmse`i'
		local corr_rmse = r(rho)
		qui pwcorr variance variance`i'
		local corr_var = r(rho)
		qui ktau rank_ambias rank_ambias`i'
		local tau_ambias = r(tau_a)
		qui ktau rank_mse rank_mse`i'
		local tau_mse = r(tau_a)
		qui ktau rank_var rank_var`i'
		local tau_var = r(tau_a)
		qui sum ambias if rank_ambias==1
		local min_ambias = r(mean)
		qui sum diff_ambias if rank_ambias`i'==1
		local diff_ambias = r(mean)
		qui sum mse if rank_mse==1
		local min_mse = r(mean)
		qui sum diff_mse if rank_mse`i'==1
		local diff_mse = r(mean)
		qui sum variance if rank_var==1
		local min_var = r(mean)
		qui sum diff_var if rank_var`i'==1
		local diff_var = r(mean)
		
		post results_placebo (`corr_mbias') (`corr_ambias') (`corr_mse') (`corr_rmse') (`corr_var') (`tau_ambias') (`tau_mse') (`tau_var') (`min_ambias') (`diff_ambias') (`min_mse') (`diff_mse') (`min_var') (`diff_var')
	}
}
postclose results_placebo



// STRUCTURED

forvalues i=1/`NSBDM' {
	drop _all
	
	foreach FN of local FNBDM {
		capture use "structured`FN'/`NBDM'_results_`i'", clear
	}
	capture keep b_* ave_*
	
	if _rc==0 {
		qui postfile biases_structured str10(estimator) mbias`i' ambias`i' mse`i' rmse`i' variance`i' using biases_structured_`i', replace
		
		foreach var of local ESTM {
			gen bias_`var' = `var' - ave_effect
			egen mbias_`var' = mean(bias_`var')
			gen ambias_`var' = abs(mbias_`var')
			
			gen biassq_`var' = (`var' - ave_effect)^2
			egen mse_`var' = mean(biassq_`var')
			gen rmse_`var' = sqrt(mse_`var')
			
			qui sum `var'
			local variance = r(Var)
			qui sum mbias_`var'
			local mbias = r(mean)
			qui sum ambias_`var'
			local ambias = r(mean)
			qui sum mse_`var'
			local mse = r(mean)
			qui sum rmse_`var'
			local rmse = r(mean)
			
			post biases_structured ("`var'") (`mbias') (`ambias') (`mse') (`rmse') (`variance')
		}
		
		postclose biases_structured
		
		use biases_structured_`i', clear
		sort ambias`i'
		gen rank_ambias`i'=_n
		sort mse`i'
		gen rank_mse`i'=_n
		sort variance`i'
		gen rank_var`i'=_n
		save biases_structured_`i', replace
	}
}

use 5_biases_true, clear
qui egen minambias = min(ambias)
qui gen diff_ambias = ambias - minambias
qui egen minmse = min(mse)
qui gen diff_mse = mse - minmse
qui egen minvar = min(variance)
qui gen diff_var = variance - minvar
drop minambias minmse minvar
forvalues i=1/`NSBDM' {
	capture merge 1:1 estimator using biases_structured_`i'
	if _rc==0 {
		drop _merge
		qui save biases_structured, replace
		erase biases_structured_`i'.dta
	}
}

use biases_structured, clear
qui postfile results_structured corr_mbias corr_ambias corr_mse corr_rmse corr_var tau_ambias tau_mse tau_var min_ambias diff_ambias min_mse diff_mse min_var diff_var using 6_results_structured, replace
forvalues i=1/`NSBDM' {
	capture novarabbrev pwcorr mbias mbias`i'
	if _rc==0 {
		qui pwcorr mbias mbias`i'
		local corr_mbias = r(rho)
		qui pwcorr ambias ambias`i'
		local corr_ambias = r(rho)
		qui pwcorr mse mse`i'
		local corr_mse = r(rho)
		qui pwcorr rmse rmse`i'
		local corr_rmse = r(rho)
		qui pwcorr variance variance`i'
		local corr_var = r(rho)
		qui ktau rank_ambias rank_ambias`i'
		local tau_ambias = r(tau_a)
		qui ktau rank_mse rank_mse`i'
		local tau_mse = r(tau_a)
		qui ktau rank_var rank_var`i'
		local tau_var = r(tau_a)
		qui sum ambias if rank_ambias==1
		local min_ambias = r(mean)
		qui sum diff_ambias if rank_ambias`i'==1
		local diff_ambias = r(mean)
		qui sum mse if rank_mse==1
		local min_mse = r(mean)
		qui sum diff_mse if rank_mse`i'==1
		local diff_mse = r(mean)
		qui sum variance if rank_var==1
		local min_var = r(mean)
		qui sum diff_var if rank_var`i'==1
		local diff_var = r(mean)
		
		post results_structured (`corr_mbias') (`corr_ambias') (`corr_mse') (`corr_rmse') (`corr_var') (`tau_ambias') (`tau_mse') (`tau_var') (`min_ambias') (`diff_ambias') (`min_mse') (`diff_mse') (`min_var') (`diff_var')
	}
}
postclose results_structured



// BOOTSTRAP

forvalues i=1/`NSBTS' {
	use `ORIG', clear
	
	foreach var of local ESTM {
		qui sum `var' if sample==`i'
		scalar `var'_orig = r(mean)
	}
	
	drop _all
	
	foreach FN of local FNBTS {
		capture use "bootstrap`FN'/`NBTS'_results_`i'", clear
	}
	capture keep b_*
	
	if _rc==0 {
		qui postfile biases_bootstrap str10(estimator) mbias`i' ambias`i' mse`i' rmse`i' variance`i' using biases_bootstrap_`i', replace
		
		foreach var of local ESTM {
			gen bias_`var' = `var' - `var'_orig
			egen mbias_`var' = mean(bias_`var')
			gen ambias_`var' = abs(mbias_`var')
			
			gen biassq_`var' = (`var' - `var'_orig)^2
			egen mse_`var' = mean(biassq_`var')
			gen rmse_`var' = sqrt(mse_`var')
			
			qui sum `var'
			local variance = r(Var)
			qui sum mbias_`var'
			local mbias = r(mean)
			qui sum ambias_`var'
			local ambias = r(mean)
			qui sum mse_`var'
			local mse = r(mean)
			qui sum rmse_`var'
			local rmse = r(mean)
			
			post biases_bootstrap ("`var'") (`mbias') (`ambias') (`mse') (`rmse') (`variance')
		}
		
		postclose biases_bootstrap
		
		use biases_bootstrap_`i', clear
		sort ambias`i'
		gen rank_ambias`i'=_n
		sort mse`i'
		gen rank_mse`i'=_n
		sort variance`i'
		gen rank_var`i'=_n
		save biases_bootstrap_`i', replace
	}
}

use 5_biases_true, clear
qui egen minambias = min(ambias)
qui gen diff_ambias = ambias - minambias
qui egen minmse = min(mse)
qui gen diff_mse = mse - minmse
qui egen minvar = min(variance)
qui gen diff_var = variance - minvar
drop minambias minmse minvar
forvalues i=1/`NSBTS' {
	capture merge 1:1 estimator using biases_bootstrap_`i'
	if _rc==0 {
		drop _merge
		qui save biases_bootstrap, replace
		erase biases_bootstrap_`i'.dta
	}
}

use biases_bootstrap, clear
qui postfile results_bootstrap corr_mbias corr_ambias corr_mse corr_rmse corr_var tau_ambias tau_mse tau_var min_ambias diff_ambias min_mse diff_mse min_var diff_var using 6_results_bootstrap, replace
forvalues i=1/`NSBTS' {
	capture novarabbrev pwcorr mbias mbias`i'
	if _rc==0 {
		qui pwcorr mbias mbias`i'
		local corr_mbias = r(rho)
		qui pwcorr ambias ambias`i'
		local corr_ambias = r(rho)
		qui pwcorr mse mse`i'
		local corr_mse = r(rho)
		qui pwcorr rmse rmse`i'
		local corr_rmse = r(rho)
		qui pwcorr variance variance`i'
		local corr_var = r(rho)
		qui ktau rank_ambias rank_ambias`i'
		local tau_ambias = r(tau_a)
		qui ktau rank_mse rank_mse`i'
		local tau_mse = r(tau_a)
		qui ktau rank_var rank_var`i'
		local tau_var = r(tau_a)
		qui sum ambias if rank_ambias==1
		local min_ambias = r(mean)
		qui sum diff_ambias if rank_ambias`i'==1
		local diff_ambias = r(mean)
		qui sum mse if rank_mse==1
		local min_mse = r(mean)
		qui sum diff_mse if rank_mse`i'==1
		local diff_mse = r(mean)
		qui sum variance if rank_var==1
		local min_var = r(mean)
		qui sum diff_var if rank_var`i'==1
		local diff_var = r(mean)
		
		post results_bootstrap (`corr_mbias') (`corr_ambias') (`corr_mse') (`corr_rmse') (`corr_var') (`tau_ambias') (`tau_mse') (`tau_var') (`min_ambias') (`diff_ambias') (`min_mse') (`diff_mse') (`min_var') (`diff_var')
	}
}
postclose results_bootstrap



// RANDOM

/*
use 5_biases_true, clear
qui egen minambias = min(ambias)
qui gen diff_ambias = ambias - minambias
qui egen minmse = min(mse)
qui gen diff_mse = mse - minmse
qui egen minvar = min(variance)
qui gen diff_var = variance - minvar
drop minambias minmse minvar
forvalues i=1/`NSRND' {
	gen random1=uniform()
	sort random1
	gen rank_ambias`i'=_n
	gen random2=uniform()
	sort random2
	gen rank_mse`i'=_n
	gen random3=uniform()
	sort random3
	gen rank_var`i'=_n
	drop random1 random2 random3
}
qui save biases_random, replace

use biases_random, clear
qui postfile results_random tau_ambias tau_mse tau_var min_ambias diff_ambias min_mse diff_mse min_var diff_var using 6_results_random, replace
forvalues i=1/`NSRND' {
	qui ktau rank_ambias rank_ambias`i'
	local tau_ambias = r(tau_a)
	qui ktau rank_mse rank_mse`i'
	local tau_mse = r(tau_a)
	qui ktau rank_var rank_var`i'
	local tau_var = r(tau_a)
	qui sum ambias if rank_ambias==1
	local min_ambias = r(mean)
	qui sum diff_ambias if rank_ambias`i'==1
	local diff_ambias = r(mean)
	qui sum mse if rank_mse==1
	local min_mse = r(mean)
	qui sum diff_mse if rank_mse`i'==1
	local diff_mse = r(mean)
	qui sum variance if rank_var==1
	local min_var = r(mean)
	qui sum diff_var if rank_var`i'==1
	local diff_var = r(mean)
		
	post results_random (`tau_ambias') (`tau_mse') (`tau_var') (`min_ambias') (`diff_ambias') (`min_mse') (`diff_mse') (`min_var') (`diff_var')
}
postclose results_random
*/

use 5_biases_true, clear

qui egen min_ambias = min(ambias)
qui egen diff_ambias = mean(ambias - min_ambias)
qui egen min_mse = min(mse)
qui egen diff_mse = mean(mse - min_mse)
qui egen min_var = min(variance)
qui egen diff_var = mean(variance - min_var)

qui gen tau_ambias = 0
qui gen tau_mse = 0
qui gen tau_var = 0

keep tau_ambias tau_mse tau_var min_ambias diff_ambias min_mse diff_mse min_var diff_var
order tau_ambias tau_mse tau_var min_ambias diff_ambias min_mse diff_mse min_var diff_var
keep in 1

save 6_results_random, replace


clear all
erase biases_placebo.dta
erase biases_structured.dta
erase biases_bootstrap.dta
*erase biases_random.dta
