clear all
set more off


use 5_biases_true, clear
gen sd=sqrt(variance)
keep estimator ambias rmse sd
sort estimator
format ambias rmse sd %30.0f

qui log using 7_results_print, t replace
di "RESULTS FOR THE ORIGINAL DGP"
list
qui log close


use 6_results_placebo, clear
gen regret_ambias = 100*(diff_ambias/min_ambias)
gen regret_mse = 100*(diff_mse/min_mse)
gen regret_var = 100*(diff_var/min_var)

qui sum diff_ambias
sca pla_diff_ambias = r(mean)
qui sum diff_mse
sca pla_diff_mse = r(mean)
qui sum diff_var
sca pla_diff_var = r(mean)

format diff* min* regret* %10.1f
format corr* tau* %10.3f

qui log using 7_results_print, t append
di _newline(2)
di "RESULTS FOR THE PLACEBO EMCS"
sum, format
qui log close


use 6_results_structured, clear
gen regret_ambias = 100*(diff_ambias/min_ambias)
gen regret_mse = 100*(diff_mse/min_mse)
gen regret_var = 100*(diff_var/min_var)

qui sum diff_ambias
sca str_diff_ambias = r(mean)
qui sum diff_mse
sca str_diff_mse = r(mean)
qui sum diff_var
sca str_diff_var = r(mean)

format diff* min* regret* %10.1f
format corr* tau* %10.3f

qui log using 7_results_print, t append
di _newline(2)
di "RESULTS FOR THE STRUCTURED EMCS"
sum, format
qui log close


use 6_results_bootstrap, clear
gen regret_ambias = 100*(diff_ambias/min_ambias)
gen regret_mse = 100*(diff_mse/min_mse)
gen regret_var = 100*(diff_var/min_var)

qui sum diff_ambias
sca boot_diff_ambias = r(mean)
qui sum diff_mse
sca boot_diff_mse = r(mean)
qui sum diff_var
sca boot_diff_var = r(mean)

format diff* min* regret* %10.1f
format corr* tau* %10.3f

qui log using 7_results_print, t append
di _newline(2)
di "RESULTS FOR THE BOOTSTRAP"
sum, format
qui log close


use 6_results_random, clear
gen regret_ambias = 100*(diff_ambias/min_ambias)
gen regret_mse = 100*(diff_mse/min_mse)
gen regret_var = 100*(diff_var/min_var)

qui sum diff_ambias
sca rndm_diff_ambias = r(mean)
qui sum diff_mse
sca rndm_diff_mse = r(mean)
qui sum diff_var
sca rndm_diff_var = r(mean)

format diff* min* regret* %10.1f
format tau* %10.3f

qui log using 7_results_print, t append
di _newline(2)
di "RESULTS FOR RANDOM RANKINGS OF ESTIMATORS"
sum, format
qui log close


capture program drop regprint
program define regprint
	di _newline(3)
	di "AVERAGE REGRET NORMALIZED BY RESULTS FOR RANDOM"
	
	di _newline
	di "PLACEBO"
	di "absolute mean bias: " round(100*(pla_diff_ambias/rndm_diff_ambias),.1)
	di "MSE: " round(100*(pla_diff_mse/rndm_diff_mse),.1)
	di "variance: " round(100*(pla_diff_var/rndm_diff_var),.1)
	
	di _newline
	di "STRUCTURED"
	di "absolute mean bias: " round(100*(str_diff_ambias/rndm_diff_ambias),.1)
	di "MSE: " round(100*(str_diff_mse/rndm_diff_mse),.1)
	di "variance: " round(100*(str_diff_var/rndm_diff_var),.1)
	
	di _newline
	di "BOOTSTRAP"
	di "absolute mean bias: " round(100*(boot_diff_ambias/rndm_diff_ambias),.1)
	di "MSE: " round(100*(boot_diff_mse/rndm_diff_mse),.1)
	di "variance: " round(100*(boot_diff_var/rndm_diff_var),.1)
	
	di _newline
	di "RANDOM"
	di "absolute mean bias: " round(100*(rndm_diff_ambias/rndm_diff_ambias),.1)
	di "MSE: " round(100*(rndm_diff_mse/rndm_diff_mse),.1)
	di "variance: " round(100*(rndm_diff_var/rndm_diff_var),.1)
end


qui log using 7_results_print, t append
regprint
qui log close


clear all
