*** This is the control file for comparing the performance of       ***
***    various nonexperimental estimators of the ATT (under CIA)    ***
*** Simulation design (pla/str): structured (Busso et al. 2014)     ***
*** Nonexperimental dataset: CPS                                    ***
*** Outcome variable (re78/u78): earnings                           ***
*** Version of the dataset (DW/ST): Dehejia & Wahba                 ***
*** Choice of control vars (bal/sim): DW (based on balancing tests) ***
*** Target group from NSW (ctr/exp): experimental group             ***
*** First version: 30/04/2018                                       ***
*** This version:  24/05/2018                                       ***
*** Authors: Arun Advani, Toru Kitagawa and Tymon Sloczynski        ***
*** Written for Stata 11                                            ***


//all the usual setup stuff
*version control
version 11

*setup standard things
capture clear all
capture log close
capture graph drop _all

*set processors 1

local NAME "structured_TS_20180524"
set seed 2018052406

*allocate memory
set mem 300m

*get rid of scroll requirement
set more off

*open log
log using `NAME', replace

*install necessary programs
capture {
	ssc install fairlie, replace
	ssc install nnmatch, replace
	ssc install oaxaca, replace
	ssc install psmatch2, replace
	ssc install moremata, replace
}


//setup locals
local LID			501 //ID of sample with lowest ID number
local HID			600 //ID of sample with highest ID number
local NREPS			500 //number of EMCS replications per sample

local NOBS			2000
local NCOBS			1500

local FOLDER		samples
local ESTIMATES		2_estimates

local OUTCOME       re78
local TREATMENT     treated
local COVARIATES    age age2 age3 educ educ2 nodegree married black hispanic re74 re75 u74 u75 re74ed
local COV1			age
local COV2			age2
local COV3			age3
local COV4			educ
local COV5			educ2
local COV6			nodegree
local COV7			married
local COV8			black
local COV9			hispanic
local COV10			re74
local COV11			re75
local COV12			u74
local COV13			u75
local COV14			re74ed
local NCOVS			15 //number of COVARIATES + 1

*************************



#delimit ;

do common_programs;

forvalues j=`LID'/`HID' {;
	
	capture erase `NAME'_all_samples_`j'.dta;
	
	use "../`ESTIMATES'", clear;
	quietly sum band_kern_u if sample==`j';
	local bw = r(sum);
	
	quietly postfile results
		b_lr b_oaxaca b_dr b_kern_u b_ipw2 b_psm1 b_psmbc1 ave_effect frac_treated N_orig N_over1 N_over2 N_T_orig N_C_orig N_T_over1 N_C_over1 N_T_over2 N_C_over2 overlap1 overlap2
		using `NAME'_results_`j', replace;
	
	timer on 2;
	
	forvalues i=1/`NREPS' {;
		quietly {;
		timer on 1;
		
		#delimit cr
		
		use "../`FOLDER'/sample`j'", clear
		
		gen u74=re74==0
		gen u75=re75==0
		gen u78=re78==0
		
		gen age2 = age^2
		gen age3 = age^3
		gen educ2 = educ^2
		gen re742 = re74^2
		gen re752 = re75^2
		gen u74h = u74*hispanic
		gen re74ed = educ*re74
		
		qui su re74 if treated==0
		scalar re74_max_c = r(max)
		scalar re74_min_c = r(min)
		
		qui su re75 if treated==0
		scalar re75_max_c = r(max)
		scalar re75_min_c = r(min)
		
		qui su re78 if treated==0
		scalar re78_max_c = r(max)
		scalar re78_min_c = r(min)
		
		qui su educ if treated==0 & nodegree==0
		scalar educ_max_c_nod0 = r(max)
		scalar educ_min_c_nod0 = r(min)
		
		qui su educ if treated==0 & nodegree==1
		scalar educ_max_c_nod1 = r(max)
		scalar educ_min_c_nod1 = r(min)
		
		qui su age if treated==0
		scalar age_max_c = r(max)
		scalar age_min_c = r(min)
		
		qui su re74 if treated==1
		scalar re74_max_t = r(max)
		scalar re74_min_t = r(min)
		
		qui su re75 if treated==1
		scalar re75_max_t = r(max)
		scalar re75_min_t = r(min)
		
		qui su re78 if treated==1
		scalar re78_max_t = r(max)
		scalar re78_min_t = r(min)
		
		qui su educ if treated==1 & nodegree==0
		scalar educ_max_t_nod0 = r(max)
		scalar educ_min_t_nod0 = r(min)
		
		qui su educ if treated==1 & nodegree==1
		scalar educ_max_t_nod1 = r(max)
		scalar educ_min_t_nod1 = r(min)
		
		qui su age if treated==1
		scalar age_max_t = r(max)
		scalar age_min_t = r(min)
		
		gen u00 = u74==0 & u75==0
		gen u01 = u74==0 & u75==1
		gen u10 = u74==1 & u75==0
		gen u11 = u74==1 & u75==1

		qui su u11			if treated==0
		scalar unemp11_0 	= r(mean)
		qui su u01			if treated==0
		scalar unemp01_0 	= r(mean) + unemp11_0
		qui su u10			if treated==0
		scalar unemp10_0 	= r(mean) + unemp01_0
		qui su u00			if treated==0
		scalar unemp00_0 	= r(mean) + unemp10_0

		qui su u11			if treated==1
		scalar unemp11_1	= r(mean)
		qui su u01			if treated==1
		scalar unemp01_1 	= r(mean) + unemp11_1
		qui su u10			if treated==1
		scalar unemp10_1 	= r(mean) + unemp01_1
		qui su u00			if treated==1
		scalar unemp00_1 	= r(mean) + unemp10_1
		
		gen lre74=ln(re74/1000)
		gen lre75=ln(re75/1000)
		gen lre78=ln(re78/1000)
		
		forvalues i_treat = 0/1 {
			forvalues i_74 = 0/1 {
				forvalues i_75 = 0/1 {
				qui su lre74	if u`i_74'`i_75'==1 & treated==`i_treat'
				scalar conditmean_re74_`i_74'`i_75'_`i_treat' = r(mean)
				scalar conditvar_re74_`i_74'`i_75'_`i_treat'  = r(Var)
				
				qui su lre75	if u`i_74'`i_75'==1 & treated==`i_treat'
				scalar conditmean_re75_`i_74'`i_75'_`i_treat' = r(mean)
				scalar conditvar_re75_`i_74'`i_75'_`i_treat'  = r(Var)
				}
			}
			qui cor lre74 lre75 if u00==1 & treated==`i_treat', c
			matrix conditcov_`i_treat' = r(C)
		}
		
		qui logit nodegree treated u74 u75 re74 re75, asis tolerance(1e-4) iterate(1000)
		mat nodegree = e(b)
		
		qui reg educ treated u74 u75 re74 re75 if nodegree==0
		mat educ_0 = e(b)
		predict res, r
		qui sum res, d
		scalar var_educ_0 = r(Var)
		drop res
		
		qui reg educ treated u74 u75 re74 re75 if nodegree==1
		mat educ_1 = e(b)
		predict res, r
		qui sum res, d
		scalar var_educ_1 = r(Var)
		drop res
		
		qui reg age educ nodegree treated u74 u75 re74 re75
		mat age = e(b)
		predict res, r
		qui sum res, d
		scalar var_age = r(Var)
		drop res
		
		qui logit married age educ nodegree treated u74 u75 re74 re75, asis tolerance(1e-4) iterate(1000)
		mat married = e(b)
		
		qui logit black married age educ nodegree treated u74 u75 re74 re75, asis tolerance(1e-4) iterate(1000)
		mat black = e(b)
		
		qui logit hispanic black married age educ nodegree treated u74 u75 re74 re75, asis tolerance(1e-4) iterate(1000)
		mat hispanic = e(b)
		
		qui logit u78 `COVARIATES' if treated==0, asis tolerance(1e-4) iterate(1000)
		mat u78_0 = e(b)
		qui logit u78 `COVARIATES' if treated==1, asis tolerance(1e-4) iterate(1000)
		mat u78_1 = e(b)
		
		qui reg re78 `COVARIATES' if treated==0
		mat re78_0 = e(b)
		predict res, r
		qui sum res, d
		scalar var_re78_0 = r(Var)
		drop res
		
		qui reg re78 `COVARIATES' if treated==1
		mat re78_1 = e(b)
		predict res, r
		qui sum res, d
		scalar var_re78_1 = r(Var)
		drop res
		
		drop _all
		
		//Create new data based on the old data
		qui set obs `NOBS'
		qui gen treated = _n>`NCOBS'
		
		gen u_temp 	= runiform()
		gen u74		= 0
		gen u75		= 0

		replace u75 = 1 if treated==0 & u_temp<unemp01_0
		replace u74	= 1 if treated==0 & u_temp<unemp11_0
		replace u74	= 1 if treated==0 & u_temp<unemp10_0 & u_temp>=unemp01_0
		
		replace u75 = 1 if treated==1 & u_temp<unemp01_1
		replace u74	= 1 if treated==1 & u_temp<unemp11_1
		replace u74	= 1 if treated==1 & u_temp<unemp10_1 & u_temp>=unemp01_1
		
		gen u00 = u74==0 & u75==0
		gen u01 = u74==0 & u75==1
		gen u10 = u74==1 & u75==0
		gen u11 = u74==1 & u75==1
		
		drop u_temp
		scalar drop unemp00_0 unemp01_0 unemp10_0 unemp11_0 unemp00_1 unemp01_1 unemp10_1 unemp11_1
		
		gen re74 = 0		//Initialise earnings as if unemployed
		gen re75 = 0
		
		//Loop to update earnings for those employed in one year only
		forvalues i_treat = 0/1 {
			replace re74 = exp(rnormal(conditmean_re74_01_`i_treat',sqrt(conditvar_re74_01_`i_treat')))*1000 if u01==1 & treated==`i_treat'
			replace re75 = exp(rnormal(conditmean_re75_10_`i_treat',sqrt(conditvar_re75_10_`i_treat')))*1000 if u10==1 & treated==`i_treat'
		}
		
		//Need a draw from joint normal; start with two standard normals which we will then correlate
		gen c1 = rnormal()
		gen c2 = rnormal()
		
		//For controls
		matrix A_0 = cholesky(conditcov_0)
		matrix colnames A_0 = c1 c2
		matrix rownames A_0 = r1 r2
		
		//Take each row of A_0 and, for each observation, multiply the values in the columns by the appropriate std normal variable (c1 c2)
		matrix		a74_0 = A_0[1,1..2]
		mat score	y74_0 = a74_0 			//Create the first of the joint normal vars
		matrix		a75_0 = A_0[2,1..2]
		mat score	y75_0 = a75_0 			//Create the second of the joint normal vars
		
		//Now create the "re" variables
		replace re74 = exp(y74_0 + conditmean_re74_00_0)*1000 if u00==1 & treated==0	//Create the "re" variables from the joint normal ones,
		replace re75 = exp(y75_0 + conditmean_re75_00_0)*1000 if u00==1 & treated==0	//	adding the appropriate mean (since y1&y2 have zero mean)
		
		//Same again for treated
		matrix A_1 = cholesky(conditcov_1)
		matrix colnames A_1 = c1 c2
		matrix rownames A_1 = r1 r2
		
		matrix		a74_1 = A_1[1,1..2]
		mat score	y74_1 = a74_1
		matrix		a75_1 = A_1[2,1..2]
		mat score	y75_1 = a75_1
		
		replace re74 = exp(y74_1 + conditmean_re74_00_1)*1000 if u00==1 & treated==1	//Create the "re" variables from the joint normal ones,
		replace re75 = exp(y75_1 + conditmean_re75_00_1)*1000 if u00==1 & treated==1	//	adding the appropriate mean (since y1&y2 have zero mean)
		
		//Clean up
		drop c1 c2 y74_0 y74_1 y75_0 y75_1
		
		//Impose the NSW-sample minima and maxima on all simulated data
		replace re74 = re74_max_c if re74>re74_max_c & treated==0
		replace re74 = re74_min_c if re74<re74_min_c & treated==0
		replace re75 = re75_max_c if re75>re75_max_c & treated==0
		replace re75 = re75_min_c if re75<re75_min_c & treated==0
		replace re74 = re74_max_t if re74>re74_max_t & treated==1
		replace re74 = re74_min_t if re74<re74_min_t & treated==1
		replace re75 = re75_max_t if re75>re75_max_t & treated==1
		replace re75 = re75_min_t if re75<re75_min_t & treated==1
		
		//Go back to generating other covariates
		mat score conditprob_nodegree = nodegree
		replace conditprob_nodegree = invlogit(conditprob_nodegree)
		gen nodegree_temp = runiform()
		gen nodegree = nodegree_temp<conditprob_nodegree
		drop nodegree_temp conditprob_nodegree
		
		mat score conditmean_educ_0 = educ_0
		gen educ_0 = rnormal(conditmean_educ_0,sqrt(var_educ_0))
		mat score conditmean_educ_1 = educ_1
		gen educ_1 = rnormal(conditmean_educ_1,sqrt(var_educ_1))
		
		gen educ = nodegree*educ_1 + (1-nodegree)*educ_0
		replace educ = educ_max_c_nod0 if educ>educ_max_c_nod0 & treated==0 & nodegree==0
		replace educ = educ_min_c_nod0 if educ<educ_min_c_nod0 & treated==0 & nodegree==0
		replace educ = educ_max_t_nod0 if educ>educ_max_t_nod0 & treated==1 & nodegree==0
		replace educ = educ_min_t_nod0 if educ<educ_min_t_nod0 & treated==1 & nodegree==0
		replace educ = educ_max_c_nod1 if educ>educ_max_c_nod1 & treated==0 & nodegree==1
		replace educ = educ_min_c_nod1 if educ<educ_min_c_nod1 & treated==0 & nodegree==1
		replace educ = educ_max_t_nod1 if educ>educ_max_t_nod1 & treated==1 & nodegree==1
		replace educ = educ_min_t_nod1 if educ<educ_min_t_nod1 & treated==1 & nodegree==1
		drop conditmean_educ_0 conditmean_educ_1 educ_0 educ_1
		
		mat score conditmean_age = age
		gen age = rnormal(conditmean_age,sqrt(var_age))
		replace age = age_max_c if age>age_max_c & treated==0
		replace age = age_min_c if age<age_min_c & treated==0
		replace age = age_max_t if age>age_max_t & treated==1
		replace age = age_min_t if age<age_min_t & treated==1
		drop conditmean_age
		
		mat score conditprob_married = married
		replace conditprob_married = invlogit(conditprob_married)
		gen married_temp = runiform()
		gen married = married_temp<conditprob_married
		drop married_temp conditprob_married
		
		mat score conditprob_black = black
		replace conditprob_black = invlogit(conditprob_black)
		gen black_temp = runiform()
		gen black = black_temp<conditprob_black
		drop black_temp conditprob_black
		
		mat score conditprob_hispanic = hispanic
		replace conditprob_hispanic = invlogit(conditprob_hispanic)
		gen hispanic_temp = runiform()
		gen hispanic = hispanic_temp<conditprob_hispanic
		drop hispanic_temp conditprob_hispanic
		
		gen age2 = age^2
		gen age3 = age^3
		gen educ2 = educ^2
		gen re742 = re74^2
		gen re752 = re75^2
		gen u74h = u74*hispanic
		gen re74ed = educ*re74
		
		//Generate the probability of employment
		mat score index78_0 = u78_0
		mat score index78_1 = u78_1
		gen index78 = treated*index78_1 + (1-treated)*index78_0
		
		gen p78 = invlogit(index78)
		gen u78_temp = runiform()
		gen u78 = u78_temp<p78
		
		//Generate the earnings variable
		mat score conditmean_re78_0 = re78_0
		gen re78_0 = rnormal(conditmean_re78_0,sqrt(var_re78_0))
		mat score conditmean_re78_1 = re78_1
		gen re78_1 = rnormal(conditmean_re78_1,sqrt(var_re78_1))
		
		gen re78 = treated*re78_1 + (1-treated)*re78_0
		replace re78 = re78_max_c if re78>re78_max_c & treated==0
		replace re78 = re78_min_c if re78<re78_min_c & treated==0
		replace re78 = re78_max_t if re78>re78_max_t & treated==1
		replace re78 = re78_min_t if re78<re78_min_t & treated==1
		
		//Calculate the "true" ATT for this sample
		gen expw0 = conditmean_re78_0
		gen expw1 = conditmean_re78_1
		gen te_i = expw1-expw0
		sum te_i if treated==1
		scalar ave_effect = r(mean)
		
		//Keep only the necessary variables
		keep `OUTCOME' `TREATMENT' `COVARIATES'
		
		save `NAME'_sample, replace
		
		local state = c(seed)
		di c(seed)
		
		#delimit ;
		
		*****************************************RUN THE ESTIMATION*********************************************;

		*record the proportion of treated in this sample (and other basic measures);
		sum `TREATMENT'; scalar frac_treated = r(mean); scalar N_orig = r(N);
		sum `TREATMENT' if `TREATMENT'==1; scalar N_T_orig = r(N);
		sum `TREATMENT' if `TREATMENT'==0; scalar N_C_orig = r(N);
		
		*estimate propensity scores;
		logit `TREATMENT' `COVARIATES', asis tolerance(1e-4) iterate(1000);
		predict double logps;
		
		*this is required by kernel matching;
		sort `TREATMENT';

		*linear regression;
		reg `OUTCOME' `TREATMENT' `COVARIATES';
		scalar b_lr = _b[`TREATMENT'];

		*linear Oaxaca-Blinder;
		capture oaxaca `OUTCOME' `COVARIATES', by(`TREATMENT') weight(1);
		if _rc==0 {;
			scalar b_oaxaca = -1*_b[unexplained];
		};
		if _rc!=0 {;
			reg `OUTCOME' `COVARIATES' if `TREATMENT'==0;
			predict pred; gen diff = `OUTCOME' - pred;
			sum diff if `TREATMENT'==1; scalar diff_mean = r(mean);
			oaxaca `OUTCOME' `COVARIATES', by(`TREATMENT') weight(1) relax;
			if round(-1*_b[unexplained],.0001)==round(diff_mean,.0001) {;
				scalar b_oaxaca = -1*_b[unexplained];
			};
			else {;
				scalar b_oaxaca = .;
			};
			drop pred diff;
		};		
		capture drop A1-A`NCOVS';
		
		*kernel matching (uniform);
		capture psmatch2 `TREATMENT', kernel out(`OUTCOME') k(uniform) p(logps) bw(`bw');
		if _rc==0 {;
			scalar b_kern_u = r(att);
		};
		if _rc!=0 {;
			scalar b_kern_u = .;
		};
		capture drop _*;
		
		*double robust regression;
		capture drobust `OUTCOME' `TREATMENT' "`COVARIATES'" no;
		if _rc==0 {;
			scalar b_dr = drob;
		};
		if _rc!=0 {;
			scalar b_dr = .;
		};
		
		*inverse probability weighting;
		capture ipw `OUTCOME' `TREATMENT' `COVARIATES';
		if _rc==0 {;
			scalar b_ipw2 = ipw2;
		};
		if _rc!=0 {;
			scalar b_ipw2 = .;
		};
		
		*matching on the estimated propensity score;
		capture nnmatch `OUTCOME' `TREATMENT' logps, tc(att) m(1);
		if _rc!=0 {;
			scalar b_psm1 = .;
		};
		if _rc==0 {;
			matrix A = e(b); svmat double A; scalar b_psm1 = A1[1]; drop A1;
		};

		*bias-corrected matching on the estimated propensity score;
		capture nnmatch `OUTCOME' `TREATMENT' logps, tc(att) m(1) biasadj(`COVARIATES');
		if _rc!=0 {;
			scalar b_psmbc1 = .;
		};
		if _rc==0 {;
			matrix A = e(b); svmat double A; scalar b_psmbc1 = A1[1]; drop A1;
		};
		
		*Calculate one measure of overlap;
		gen within=.; comsup2 `TREATMENT' within;
		sum `TREATMENT' if within; scalar N_over1 = r(N); scalar overlap1 = N_over1/N_orig;
		sum `TREATMENT' if within==1 & `TREATMENT'==1; scalar N_T_over1 = r(N);
		sum `TREATMENT' if within==1 & `TREATMENT'==0; scalar N_C_over1 = r(N);
		
		*Calculate another measure of overlap;
		comsup `TREATMENT';
		sum `TREATMENT'; scalar N_over2 = r(N); scalar overlap2 = N_over2/N_orig;
		sum `TREATMENT' if `TREATMENT'==1; scalar N_T_over2 = r(N);
		sum `TREATMENT' if `TREATMENT'==0; scalar N_C_over2 = r(N);
		
		set seed `state';
		
		
		//STORE THE OUTPUT;
		post results
			(b_lr) (b_oaxaca) (b_dr) (b_kern_u) (b_ipw2) (b_psm1) (b_psmbc1)
			(ave_effect) (frac_treated) (N_orig) (N_over1) (N_over2) (N_T_orig) (N_C_orig) (N_T_over1) (N_C_over1) (N_T_over2) (N_C_over2) (overlap1) (overlap2);
		
		timer off 1;
		noisily timer list 1;
		
		capture drop _all;
		
		capture use `NAME'_all_samples_`j', clear;
		append using `NAME'_sample;
		keep `OUTCOME' `TREATMENT' age educ nodegree married black hispanic re74 re75;
		save `NAME'_all_samples_`j', replace;
		};
	};
	timer clear 1;
	
	timer off 2;
	noisily display in green "Finished with sample " in yel `j';
	noisily timer list 2;
	noisily display _newline(1);
	
	postclose results;
	capture drop _all;
	erase `NAME'_sample.dta;
};

log close;
